@@ -73,8 +73,6 @@
 #define KEY_BELL_DURATION  "bell-duration"
 #define KEY_BELL_MODE      "bell-mode"
 
-#define KEY_SWITCHER "input-sources-switcher"
-
 #define GNOME_DESKTOP_INTERFACE_DIR "org.gnome.desktop.interface"
 
 #define KEY_GTK_IM_MODULE    "gtk-im-module"
@@ -93,6 +91,9 @@
 #define DEFAULT_LANGUAGE "en_US"
 #define DEFAULT_LAYOUT "us"
 
+#define GSD_KEYBOARD_DBUS_NAME  "org.gnome.SettingsDaemon.Keyboard"
+#define GSD_KEYBOARD_DBUS_PATH  "/org/gnome/SettingsDaemon/Keyboard"
+
 struct GsdKeyboardManagerPrivate
 {
 	guint      start_idle_id;
@@ -113,8 +114,13 @@ struct GsdKeyboardManagerPrivate
         guint device_added_id;
         guint device_removed_id;
 
-        gboolean input_sources_switcher_spawned;
-        GPid input_sources_switcher_pid;
+        GDBusConnection *dbus_connection;
+        GDBusNodeInfo *dbus_introspection;
+        guint dbus_own_name_id;
+        guint dbus_register_object_id;
+
+        GDBusMethodInvocation *invocation;
+        guint pending_ops;
 };
 
 static void     gsd_keyboard_manager_class_init  (GsdKeyboardManagerClass *klass);
@@ -126,9 +132,20 @@ static gboolean apply_input_sources_settings     (GSettings               *setti
                                                   GsdKeyboardManager      *manager);
 static void     set_gtk_im_module                (GsdKeyboardManager      *manager,
                                                   GVariant                *sources);
+static void     maybe_return_from_set_input_source (GsdKeyboardManager    *manager);
+static void     increment_set_input_source_ops   (GsdKeyboardManager      *manager);
 
 G_DEFINE_TYPE (GsdKeyboardManager, gsd_keyboard_manager, G_TYPE_OBJECT)
 
+static const gchar introspection_xml[] =
+        "<node>"
+        "  <interface name='org.gnome.SettingsDaemon.Keyboard'>"
+        "    <method name='SetInputSource'>"
+        "      <arg type='u' name='idx' direction='in'/>"
+        "    </method>"
+        "  </interface>"
+        "</node>";
+
 static gpointer manager_object = NULL;
 
 static void
@@ -275,7 +292,10 @@ set_ibus_engine_finish (GObject            *object,
                 if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
                         g_warning ("Couldn't set IBus engine: %s", error->message);
                 g_error_free (error);
+                return;
         }
+
+        maybe_return_from_set_input_source (manager);
 }
 
 static void
@@ -291,6 +311,8 @@ set_ibus_engine (GsdKeyboardManager *manager,
         g_clear_object (&priv->ibus_cancellable);
         priv->ibus_cancellable = g_cancellable_new ();
 
+        increment_set_input_source_ops (manager);
+
         ibus_bus_set_global_engine_async (priv->ibus,
                                           engine_id,
                                           -1,
@@ -1034,6 +1056,7 @@ apply_input_sources_settings (GSettings          *settings,
 
  exit:
         apply_xkb_settings (manager, layout, variant, options);
+        maybe_return_from_set_input_source (manager);
         g_variant_unref (sources);
         g_free (layout);
         g_free (variant);
@@ -1149,47 +1172,6 @@ apply_all_settings (GsdKeyboardManager *manager)
 	apply_numlock (manager);
 }
 
-static void
-set_input_sources_switcher (GsdKeyboardManager *manager,
-                            gboolean            state)
-{
-        if (state) {
-                GError *error = NULL;
-                char *args[2];
-
-                if (manager->priv->input_sources_switcher_spawned)
-                        set_input_sources_switcher (manager, FALSE);
-
-                args[0] = LIBEXECDIR "/gsd-input-sources-switcher";
-                args[1] = NULL;
-
-                g_spawn_async (NULL, args, NULL,
-                               0, NULL, NULL,
-                               &manager->priv->input_sources_switcher_pid, &error);
-
-                manager->priv->input_sources_switcher_spawned = (error == NULL);
-
-                if (error) {
-                        g_warning ("Couldn't spawn %s: %s", args[0], error->message);
-                        g_error_free (error);
-                }
-        } else if (manager->priv->input_sources_switcher_spawned) {
-                kill (manager->priv->input_sources_switcher_pid, SIGHUP);
-                g_spawn_close_pid (manager->priv->input_sources_switcher_pid);
-                manager->priv->input_sources_switcher_spawned = FALSE;
-        }
-}
-
-static gboolean
-enable_switcher (GsdKeyboardManager *manager)
-{
-        GsdInputSourcesSwitcher switcher;
-
-        switcher = g_settings_get_enum (manager->priv->settings, KEY_SWITCHER);
-
-        return switcher != GSD_INPUT_SOURCES_SWITCHER_OFF;
-}
-
 static void
 settings_changed (GSettings          *settings,
                   const char         *key,
@@ -1212,8 +1194,6 @@ settings_changed (GSettings          *settings,
 		 g_strcmp0 (key, KEY_DELAY) == 0) {
 		g_debug ("Key repeat setting '%s' changed, applying key repeat settings", key);
 		apply_repeat (manager);
-        } else if (g_strcmp0 (key, KEY_SWITCHER) == 0) {
-                set_input_sources_switcher (manager, enable_switcher (manager));
 	} else {
 		g_warning ("Unhandled settings change, key '%s'", key);
 	}
@@ -1365,13 +1345,8 @@ convert_libgnomekbd_options (GSettings *settings)
                 gchar **strv;
 
                 strv = g_strsplit (*o, "\t", 2);
-                if (strv[0] && strv[1]) {
-                        /* We don't want the group switcher because
-                         * it's incompatible with the way we use XKB
-                         * groups. */
-                        if (!g_str_has_prefix (strv[1], "grp:"))
-                                g_ptr_array_add (opt_array, g_strdup (strv[1]));
-                }
+                if (strv[0] && strv[1])
+                        g_ptr_array_add (opt_array, g_strdup (strv[1]));
                 g_strfreev (strv);
         }
         g_ptr_array_add (opt_array, NULL);
@@ -1504,6 +1479,156 @@ maybe_create_initial_settings (GsdKeyboardManager *manager)
         g_strfreev (options);
 }
 
+static void
+set_input_source_return (GDBusMethodInvocation *invocation)
+{
+        g_dbus_method_invocation_return_value (invocation, NULL);
+}
+
+static void
+maybe_return_from_set_input_source (GsdKeyboardManager *manager)
+{
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+
+        if (!priv->invocation)
+                return;
+
+        if (priv->pending_ops > 0) {
+                priv->pending_ops -= 1;
+                return;
+        }
+
+        g_clear_pointer (&priv->invocation, set_input_source_return);
+}
+
+static void
+increment_set_input_source_ops (GsdKeyboardManager *manager)
+{
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+
+        if (!priv->invocation)
+                return;
+
+        priv->pending_ops += 1;
+}
+
+static void
+set_input_source (GsdKeyboardManager *manager)
+{
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+        guint idx;
+
+        g_variant_get (g_dbus_method_invocation_get_parameters (priv->invocation), "(u)", &idx);
+
+        if (idx == g_settings_get_uint (priv->input_sources_settings, KEY_CURRENT_INPUT_SOURCE)) {
+                maybe_return_from_set_input_source (manager);
+                return;
+        }
+
+        g_settings_set_uint (priv->input_sources_settings, KEY_CURRENT_INPUT_SOURCE, idx);
+}
+
+static void
+handle_dbus_method_call (GDBusConnection       *connection,
+                         const gchar           *sender,
+                         const gchar           *object_path,
+                         const gchar           *interface_name,
+                         const gchar           *method_name,
+                         GVariant              *parameters,
+                         GDBusMethodInvocation *invocation,
+                         GsdKeyboardManager    *manager)
+{
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+
+        if (g_str_equal (method_name, "SetInputSource")) {
+                if (priv->invocation) {
+#ifdef HAVE_IBUS
+                        /* This can only happen if there's an
+                         * ibus_bus_set_global_engine_async() call
+                         * going on. */
+                        g_cancellable_cancel (priv->ibus_cancellable);
+#endif
+                        g_clear_pointer (&priv->invocation, set_input_source_return);
+                        priv->pending_ops = 0;
+                }
+                priv->invocation = invocation;
+                set_input_source (manager);
+        }
+}
+
+static void
+on_bus_name_lost (GDBusConnection *connection,
+                  const gchar     *name,
+                  gpointer         data)
+{
+        g_warning ("DBus name %s lost", name);
+}
+
+static void
+got_session_bus (GObject            *source,
+                 GAsyncResult       *res,
+                 GsdKeyboardManager *manager)
+{
+        GsdKeyboardManagerPrivate *priv;
+        GDBusConnection *connection;
+        GError *error = NULL;
+        GDBusInterfaceVTable vtable = {
+                (GDBusInterfaceMethodCallFunc) handle_dbus_method_call,
+                NULL,
+                NULL,
+        };
+
+        connection = g_bus_get_finish (res, &error);
+        if (!connection) {
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Couldn't get session bus: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        priv = manager->priv;
+        priv->dbus_connection = connection;
+
+        priv->dbus_register_object_id = g_dbus_connection_register_object (priv->dbus_connection,
+                                                                           GSD_KEYBOARD_DBUS_PATH,
+                                                                           priv->dbus_introspection->interfaces[0],
+                                                                           &vtable,
+                                                                           manager,
+                                                                           NULL,
+                                                                           &error);
+        if (!priv->dbus_register_object_id) {
+                g_warning ("Error registering object: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        priv->dbus_own_name_id = g_bus_own_name_on_connection (priv->dbus_connection,
+                                                               GSD_KEYBOARD_DBUS_NAME,
+                                                               G_BUS_NAME_OWNER_FLAGS_NONE,
+                                                               NULL,
+                                                               on_bus_name_lost,
+                                                               NULL,
+                                                               NULL);
+}
+
+static void
+register_manager_dbus (GsdKeyboardManager *manager)
+{
+        GError *error = NULL;
+
+        manager->priv->dbus_introspection = g_dbus_node_info_new_for_xml (introspection_xml, &error);
+        if (error) {
+                g_warning ("Error creating introspection data: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        g_bus_get (G_BUS_TYPE_SESSION,
+                   manager->priv->cancellable,
+                   (GAsyncReadyCallback) got_session_bus,
+                   manager);
+}
+
 static void
 localed_proxy_ready (GObject      *source,
                      GAsyncResult *res,
@@ -1528,6 +1653,7 @@ localed_proxy_ready (GObject      *source,
         maybe_create_initial_settings (manager);
 out:
         apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
+        register_manager_dbus (manager);
 }
 
 static gboolean
@@ -1569,7 +1695,6 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
                           G_CALLBACK (apply_input_sources_settings), manager);
 
 	install_xkb_filter (manager);
-        set_input_sources_switcher (manager, enable_switcher (manager));
 
         gnome_settings_profile_end (NULL);
 
@@ -1603,6 +1728,17 @@ gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
 
         g_debug ("Stopping keyboard manager");
 
+        if (p->dbus_own_name_id) {
+                g_bus_unown_name (p->dbus_own_name_id);
+                p->dbus_own_name_id = 0;
+        }
+
+        if (p->dbus_register_object_id) {
+                g_dbus_connection_unregister_object (p->dbus_connection,
+                                                     p->dbus_register_object_id);
+                p->dbus_register_object_id = 0;
+        }
+
         g_cancellable_cancel (p->cancellable);
         g_clear_object (&p->cancellable);
 
@@ -1624,7 +1760,9 @@ gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
 
 	remove_xkb_filter (manager);
 
-        set_input_sources_switcher (manager, FALSE);
+        g_clear_pointer (&p->invocation, set_input_source_return);
+        g_clear_pointer (&p->dbus_introspection, g_dbus_node_info_unref);
+        g_clear_object (&p->dbus_connection);
 }
 
 static void
